//---------------------------------------------------------------------------
// File: TMainForm.cpp, TDx_Library, How the Components Work - Part 2
//
// Date: 16th January, 2004
// Author: Darren John Dwyer
// Copyright: (c) 2002-2004 Darren John Dwyer, All Rights Reserved.
//
// Description:
// The TDx_Library interface components have OnCreate() and OnDestroy() events,
// which you can use to easily structure code for automatic cascading of DirectX objects.
//
// This application shows how to cascade the creation and destruction of associated directx elements
// using the OnCreate() and OnDestroy() events.
//
// It creates an interface to DirectSound, then creates a DirectSoundBuffer from a .WAV file,
// starts playing the .wav, waits while the .wav is playing, destroys the DirectSoundBuffer,
// and finally destroys the DirectSound interface :-
//
//---------------------------------------------------------------------------
#include <vcl\vcl.h>
//---------------------------------------------------------------------------
#pragma hdrstop
//---------------------------------------------------------------------------
#include "TMainForm.h"
//---------------------------------------------------------------------------
#if (__BORLANDC__ >= 0x0530) // BCB Version 3+
    #pragma package(smart_init)
#endif
//---------------------------------------------------------------------------
#pragma link "TDx_Sound"
#pragma link "TDx_SoundBuffer"
#pragma link "TDSBufferDesc"
#pragma link "TDx_Sound_Library_Install"
//---------------------------------------------------------------------------
#pragma resource "*.dfm"
//---------------------------------------------------------------------------
#define Msg(A)  {\
                if (StatusMemo->Text.Length()>30000) StatusMemo->Text = "";\
                StatusMemo->Lines->Add(A);\
                }
//---------------------------------------------------------------------------
TMainForm *MainForm;
//---------------------------------------------------------------------------
__fastcall TMainForm::TMainForm(TComponent* Owner)
    : TForm(Owner)
{
}
//---------------------------------------------------------------------------
__fastcall TMainForm::~TMainForm()
{
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Sound1Create(TObject *Sender)
{
Msg("DirectSound interface has been created, creating DirectSoundBuffer...");
Dx_SoundBuffer1->CreateFromFile("TDx_Library.wav", DSBufferDesc1, Dx_Sound1);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Sound1Destroy(TObject *Sender)
{
Msg("DirectSound interface is about to be destroyed, may need to destroy DirectSoundBuffer...");
if (Dx_SoundBuffer1->Created) Dx_SoundBuffer1->Destroy();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Sound1Error(TObject *Sender, AnsiString Function,
	AnsiString Error, AnsiString ErrorMessage)
{
Msg( "An error occurred.");
Msg( "Function: "+Function );
Msg( "Error: "+Error );
Msg( "Error code: "+IntToStr( (int) Dx_Sound1->ErrorValue ) );
Msg( "Error message: "+ErrorMessage );
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_SoundBuffer1Create(TObject *Sender)
{
Msg("DirectSoundBuffer has been created, setting cooperative level, playing sound...");

Dx_Sound1->SetCooperativeLevel( Handle, DSSCL_PRIORITY );
Dx_SoundBuffer1->Play(0,0);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_SoundBuffer1Destroy(TObject *Sender)
{
Msg("DirectSoundBuffer is about to be destroyed, stopping if necessary.");
if (Dx_SoundBuffer1->Playing) Dx_SoundBuffer1->Stop();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_SoundBuffer1Error(TObject *Sender,
	AnsiString Function, AnsiString Error, AnsiString ErrorMessage)
{
Msg( "An error occurred.");
Msg( "Function: "+Function );
Msg( "Error: "+Error );
Msg( "Error code: "+IntToStr( (int) Dx_SoundBuffer1->ErrorValue ) );
Msg( "Error message: "+ErrorMessage );
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_SoundBuffer1BufferLost(TObject *Sender)
{
Msg("DirectSoundBuffer was lost, recreating as necessary");
Dx_SoundBuffer1->CreateFromFile("TDx_Library.wav", DSBufferDesc1, Dx_Sound1);
Dx_SoundBuffer1->Play(0,0);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormDestroy(TObject *Sender)
{
if (Dx_Sound1->Created) Dx_Sound1->Destroy();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::TestButtonClick(TObject *Sender)
{
Msg( "Test button was clicked, playing an example .WAV file..." );

Dx_Sound1->Create(NULL);
if (Dx_SoundBuffer1->Created) Msg("DirectSoundBuffer is playing...");
while (Dx_SoundBuffer1->Playing) ; // wait for sound to finish playing
if (Dx_SoundBuffer1->Created) Msg("DirectSoundBuffer has stopped playing");
if (Dx_Sound1->Created) Dx_Sound1->Destroy();

Msg( "...Finished playing the example .WAV file" );
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::QuitButtonClick(TObject *Sender)
{
Close();
}
//---------------------------------------------------------------------------
